﻿using System;
using System.IO;
using System.Linq;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Problems.Instances;
using HeuristicLab.Problems.Instances.Views;

namespace HeuristicLab.Problems.GPDL.Views {
  public partial class GpdlEditor : AsynchronousContentView, IProblemInstanceConsumer<string> {
    public GpdlEditor() {
      InitializeComponent();

      viewHost.Content = null; //necessary to enable the change of the ViewType
      viewHost.ViewType = typeof(GpdlProblemInstanceConsumerView);
      viewHost.Content = this;
      ProblemInstanceConsumerView view = (ProblemInstanceConsumerView)viewHost.ActiveView;
      splitContainer.Panel1Collapsed = !view.ProblemInstanceProviders.Any();
    }

    private void compileButton_Click(object sender, EventArgs e) {
      Utils.InstallModule("Utils", new Utils.ModuleMethodDelegate(Utils.UtilsMethod));
      Utils.InstallModule("Sets", new Utils.ModuleMethodDelegate(Sets.SetsMethod));
      Utils.InstallModule("Errors", new Utils.ModuleMethodDelegate(Errors.ErrorsMethod));

      Utils.InstallModule("GPDefLex", new Utils.ModuleMethodDelegate(GPDefLex.GPDefLexMethod));
      Utils.InstallModule("GPDefSem", new Utils.ModuleMethodDelegate(GPDefSem.GPDefSemMethod));
      Utils.InstallModule("GPDefSyn", new Utils.ModuleMethodDelegate(GPDefSyn.GPDefSynMethod));

      // --- initialize modules ---
      Utils.Modules(Utils.ModuleAction.initModule);

      Errors.PushAbortMethod(new Errors.AbortMethod(Abort));

      using (var src = new StringReader(gpdlTextBox.Text)) {
        GPDefLex.src = src;
        GPDefSyn.Parse();
      }

      GPDefLex.InitLex();
      GPDefSyn.Interpret();

      MainForm.MainFormManager.MainForm.ShowContent(GPDefSem.problem);
    }

    private void Abort(string abortKind, string moduleName, string methodName, string descr) {

    }

    void IProblemInstanceConsumer<string>.Load(string data) {
      this.gpdlTextBox.Text = data;
    }

  }
}
